from .Attendance import Attendance
from .AttendanceCode import AttendanceCode
from .CalendarDay import CalendarDay
from .Course import Course
from .CC import CC
from .Course import Course
from .CycleDay import CycleDay
from .Period import Period
from .School import School
from .SchoolCourse import SchoolCourse
from .Section import Section
from .SectionMeeting import SectionMeeting
from .Student import Student
from .StudentCoreField import StudentCoreField
from .StudentTest import StudentTest
from .StudentTestScore import StudentTestScore
from .Teacher import Teacher
from .Term import Term
from .Test import Test
from .TestScore import TestScore



from powerschool_alchemy.db import create_session
from powerschool_alchemy.models import Section, CalendarDay, CC, CycleDay
sess = create_session()
my_section = sess.query(Section).filter(Section.id==149618).first()

section_cal_days = sess \
            .query(CalendarDay, CycleDay.letter) \
            .join(CycleDay) \
            .filter(
                CalendarDay.school_id==my_section.school_id, 
                CalendarDay.date_value >= my_section.date_enrolled, 
                CalendarDay.date_value < my_section.date_left,
                CycleDay.letter.in_(
                    list(map(lambda x: x.cycle_day_letter, my_section.section_meetings)))) \
            .all()
print(section_cal_days)


from pprint import pformat
from sqlalchemy import (Boolean, ForeignKeyConstraint, Column, String, Date, Integer)
from sqlalchemy.orm import relationship

from .base import Base


class CycleDay(Base):
    __tablename__ = 'cycle_day'
    __table_args__ = (
        ForeignKeyConstraint(
            ['schoolid'],
            ['schools.school_number']
        ),
    )

    dcid = Column(Integer, primary_key=True)
    id = Column(Integer, unique=True)
    school_id = Column('schoolid', Integer)
    year_id = Column(Integer)
    letter = Column(String)
    day_number = Column(Integer)
    abbreviation = Column(String)
    day_name = Column(String)
    sort_order = Column('sortorder', Integer)

    def __repr__(self):
        return 'CycleDay: ' + pformat(vars(self))


from pprint import pformat

from sqlalchemy import (Column, ForeignKeyConstraint, Integer, String)
from sqlalchemy.orm import relationship

from .base import Base


class SectionMeeting(Base):
    __tablename__ = 'section_meeting'
    __table_args__ = (
        ForeignKeyConstraint(
            ['schoolid'],
            ['schools.school_number'],
        ),
        ForeignKeyConstraint(
            ['sectionid'],
            ['sections.id'],
        ),
        ForeignKeyConstraint(
            ['schoolid', 'cycle_day_letter', 'year_id'],
            ['cycle_day.schoolid', 'cycle_day.letter', 'cycle_day.year_id']
        )
    )

    dcid = Column(Integer, primary_key=True)
    school_id = Column('schoolid', Integer)
    school = relationship('School')
    year_id = Column(Integer)
    section_id = Column('sectionid', Integer)
    period_number = Column(Integer)
    cycle_day_letter = Column(String)
    section = relationship(
        'Section', back_populates='section_meetings')
    cycle_day = relationship('CycleDay', viewonly=True)

    def __repr__(self):
        return 'SectionMeeting: ' + pformat(vars(self))



from pprint import pformat

from sqlalchemy import (Column, Date, ForeignKeyConstraint, Integer, String)
from sqlalchemy.orm import relationship

from .base import Base


class CC(Base):
    __tablename__ = 'cc'
    __table_args__ = (
        ForeignKeyConstraint(
            ['studentid'],
            ['students.id']
        ),
        ForeignKeyConstraint(
            ['schoolid'],
            ['schools.school_number']
        ),
        ForeignKeyConstraint(
            ['termid', 'schoolid'],
            ['terms.id', 'terms.schoolid']
        ),
    )

    dcid = Column(Integer, primary_key=True)
    id = Column(Integer, unique=True)
    student_id = Column('studentid', Integer)
    student = relationship('Student')
    section_id = Column('sectionid', Integer)
    # The CC.section_id value is negated when a student drops the course
    # primaryjoin here checks for sectionids that were negated
    section = relationship('Section', viewonly=True, primaryjoin="and_("
                                                                "or_(Section.id==CC.section_id, Section.id==-CC.section_id), "
                                                                "Term.school_id==CC.school_id)")
    course_number = Column(String)
    date_enrolled = Column('dateenrolled', Date)
    date_left = Column('dateleft', Date)
    school_id = Column('schoolid', Integer)
    school = relationship('School')
    term_id = Column('termid', Integer)
    # The CC.term_id value is negated when a student drops the course
    # primaryjoin here checks for termids that were negated
    term = relationship('Term', viewonly=True, primaryjoin="and_("
                                                                "or_(Term.id==CC.term_id, Term.id==-CC.term_id), "
                                                                "Term.school_id==CC.school_id)")
    current_absences = Column('currentabsences', Integer)
    current_tardies = Column('currenttardies', Integer)
    teacher_id = Column('teacherid', Integer)
    expression = Column(String)

    def __repr__(self):
        return 'CC: ' + pformat(vars(self))


from pprint import pformat

from sqlalchemy import (Column, Integer, String)

from .base import Base


class AttendanceCode(Base):
    __tablename__ = 'attendance_code'

    dcid = Column(Integer, primary_key=True)
    id = Column(Integer, unique=True)
    presence_status_code = Column('presence_status_cd', String)
    attendance_code = Column('att_code', String)

    def __repr__(self):
        return 'Attendance Code: ' + pformat(vars(self))



from pprint import pformat
from sqlalchemy import (Boolean, ForeignKeyConstraint, Column, Date, Integer)
from sqlalchemy.orm import relationship

from .base import Base


class CalendarDay(Base):
    __tablename__ = 'calendar_day'
    __table_args__ = (
        ForeignKeyConstraint(
            ['schoolid'],
            ['schools.school_number']
        ),
        ForeignKeyConstraint(
            ['cycle_day_id'],
            ['cycle_day.id'],
        ),
    )

    id = Column(Integer)
    dcid = Column(Integer, primary_key=True)
    date_value = Column(Date)
    school_id = Column('schoolid', Integer)
    in_session = Column('insession', Boolean)
    cycle_day_id = Column(Integer)
    cycle_day = relationship('CycleDay', viewonly=True)

    school = relationship('School')

    def __repr__(self):
            return 'CalendarDay: ' + pformat(vars(self))



from pprint import pformat

from sqlalchemy import (Column, ForeignKeyConstraint, Integer)
from sqlalchemy.orm import relationship

from .base import Base


class StudentTest(Base):
    __tablename__ = 'studenttest'
    __table_args__ = (
        ForeignKeyConstraint(
            ['termid', 'schoolid'],
            ['terms.id', 'terms.schoolid']
        ),
        ForeignKeyConstraint(
            ['studentid'],
            ['students.id']
        ),
        ForeignKeyConstraint(
            ['testid'],
            ['test.id']
        ),
    )

    id = Column(Integer)
    dcid = Column(Integer, primary_key=True)
    term_id = Column('termid', Integer)
    test_id = Column('testid', Integer)
    school_id = Column('schoolid', Integer)
    student_id = Column('studentid', Integer)

    term = relationship('Term')
    student = relationship('Student')
    test = relationship('Test')
    student_test_scores = relationship(
        'StudentTestScore', back_populates='student_test')

    def __repr__(self):
        return 'StudentTest: ' + pformat(vars(self))



from pprint import pformat

from sqlalchemy import (Column, Float, ForeignKeyConstraint, Integer, String)
from sqlalchemy.orm import relationship

from .base import Base


class Course(Base):
    __tablename__ = 'courses'
    __table_args__ = (
        ForeignKeyConstraint(
            ['schoolid'],
            ['schools.school_number']
        ),
    )

    id = Column(Integer)
    dcid = Column(Integer, primary_key=True)
    name = Column('course_name', String)
    code = Column(String)
    number = Column('course_number', String)
    credit_hours = Column(Float)
    school_id = Column('schoolid', Integer)

    school = relationship('School')
    school_courses = relationship('SchoolCourse')

    @classmethod
    def get_alt_course_number(cls, course_id, session) -> str:
        alt_course_number = session.execute("""SELECT ps_customfields.getcoursescf(id, 'alt_course_number') 
                                            FROM courses 
                                            WHERE courses.id = :course_id""", 
                                        {'course_id': course_id}) \
                                .fetchall()
        alt_course_number_result = list(alt_course_number)
        # first [0] refers to the first row. We only expect one row in the result
        # second [0] refers to the alt_course_number field. If that field is blank, 
        # it will be set to None.
        if alt_course_number_result[0][0]:
            return alt_course_number_result[0][0]
        else:
            return ''


    def __repr__(self):
        return 'Course: ' + pformat(vars(self))



from pprint import pformat

from sqlalchemy import (Column, ForeignKeyConstraint, Integer, String)
from sqlalchemy.orm import relationship

from .base import Base


class Teacher(Base):
    __tablename__ = 'teachers'
    __table_args__ = (
        ForeignKeyConstraint(
            ['schoolid'],
            ['schools.school_number']
        ),
    )

    dcid = Column(Integer, primary_key=True)
    id = Column(Integer)
    last_first = Column('lastfirst', String)
    first_name = Column(String)
    middle_name = Column(String)
    last_name = Column(String)
    school_id = Column('schoolid', Integer)
    school = relationship('School')
    status = Column(Integer)
    email_address = Column('email_addr', String)

    def __repr__(self):
        return 'Teacher: ' + pformat(vars(self))



from pprint import pformat

from sqlalchemy import (Column, Integer, String)
from sqlalchemy.orm import relationship
from .base import Base


class StudentCoreField(Base):
    __tablename__ = 'studentcorefields'

    student_dcid = Column('studentsdcid', Integer, primary_key=True)
    student = relationship('Student', back_populates='core_fields')
    father_home_phone = Column(String(length=4000))
    father_day_phone = Column('fatherdayphone', String(length=4000))
    mother_home_phone = Column(String(length=4000))
    mother_day_phone = Column('motherdayphone', String(length=4000))

    def __repr__(self):
            return 'StudentCoreField: ' + pformat(vars(self))
